@@ -9,7 +9,6 @@ import Adw from 'gi://Adw';
 import Gio from 'gi://Gio';
 import GLib from 'gi://GLib';
 import GObject from 'gi://GObject';
-import Gtk from 'gi://Gtk?version=4.0';
 import Polari from 'gi://Polari';
 import Tp from 'gi://TelepathyGLib';
 
@@ -18,6 +17,7 @@ import {setConsoleLogDomain} from 'console';
 import AccountsMonitor from './accountsMonitor.js';
 import * as Connections from './connections.js';
 import InitialSetupWindow from './initialSetup.js';
+import * as Logger from './logger.js';
 import MainWindow from './mainWindow.js';
 import NetworksManager from './networksManager.js';
 import PasteManager from './pasteManager.js';
@@ -169,6 +169,39 @@ class Application extends Adw.Application {
         }
     }
 
+    async _maybeImportLogs() {
+        const path = GLib.build_filenamev(
+            [GLib.get_user_data_dir(), 'polari', 'chatlogs.v1']);
+        const file = Gio.File.new_for_path(path);
+        if (file.query_exists(null))
+            return;
+
+        try {
+            this.hold();
+            this._importingLogs = true;
+
+            let importer = new Logger.LogImporter();
+
+            let numFiles = await importer.init();
+            let n = 0;
+            this.activeWindow?.showImportProgress(n, numFiles);
+
+            // eslint-disable-next-line no-await-in-loop
+            while (await importer.importNext()) {
+                this.activeWindow?.showImportProgress(n, numFiles);
+                n++;
+            }
+
+            this.activeWindow?.showImportProgress(n, numFiles);
+        } catch (e) {
+            console.error('Failed to import telepathy-logger logs');
+            console.debug(e);
+        } finally {
+            this.release();
+            this._importingLogs = false;
+        }
+    }
+
     _maybeMakeNonUnique() {
         let bus = Gio.BusType.SESSION;
         let name = this.application_id;
@@ -207,11 +240,6 @@ class Application extends Adw.Application {
             `${Tp.CM_OBJECT_PATH_BASE}idle`,
             'org.freedesktop.Telepathy.ConnectionManager',
             '/app/libexec/telepathy-idle');
-        this._ensureService(conn,
-            `${Tp.CLIENT_BUS_NAME_BASE}Logger`,
-            `${Tp.CLIENT_OBJECT_PATH_BASE}Logger`,
-            Tp.IFACE_CLIENT,
-            '/app/libexec/telepathy-logger');
         return true;
     }
 
@@ -401,6 +429,8 @@ class Application extends Adw.Application {
         this.activate_action('start-client', null);
 
         if (!this.active_window) {
+            this._maybeImportLogs();
+
             if (this._needsInitialSetup()) {
                 new InitialSetupWindow({application: this});
             } else {
@@ -562,6 +592,10 @@ class Application extends Adw.Application {
         return this.flags & Gio.ApplicationFlags.NON_UNIQUE;
     }
 
+    get importingLogs() {
+        return this._importingLogs;
+    }
+
     _updateUserListAction() {
         let room = this.active_window.active_room;
         let action = this.lookup_action('user-list');
@@ -791,9 +825,8 @@ class Application extends Adw.Application {
     _onEditConnection(action, parameter) {
         let accountPath = parameter.deep_unpack();
         let account = this._accountsMonitor.lookupAccount(accountPath);
-        let dialog = new Connections.ConnectionProperties(account);
-        dialog.transient_for = this.active_window;
-        dialog.show();
+        const dialog = new Connections.ConnectionProperties(account);
+        dialog.present(this.activeWindow);
     }
 
     _onStartClient() {
@@ -813,9 +846,11 @@ class Application extends Adw.Application {
     }
 
     _onShowAbout() {
-        let aboutParams = {
-            application_name: _('Polari'),
-            developer_name: 'Florian Müllner',
+        const [version] = pkg.version.split('-');
+        const aboutDialog = Adw.AboutDialog.new_from_appdata(
+            '/org/gnome/Polari/metainfo.xml', version);
+
+        aboutDialog.set({
             developers: [
                 'Florian Müllner <fmuellner@gnome.org>',
                 'William Jon McCann <william.jon.mccann@gmail.com>',
@@ -848,27 +883,10 @@ class Application extends Adw.Application {
                 'Allan Day <allanpday@gmail.com>',
             ],
             translator_credits: _('translator-credits'),
-            copyright: 'Copyright © 2013-2018 The Polari authors',
-            license_type: Gtk.License.GPL_2_0,
-            application_icon: 'org.gnome.Polari',
-            version: pkg.version,
-            issue_url: 'https://gitlab.gnome.org/GNOME/polari/-/issues',
-            website: 'https://apps.gnome.org/Polari',
-            release_notes: `
-              <ul>
-                <li>Adapt to latest GNOME design pattern</li>
-                <li>Improve visuals</li>
-                <li>Remove GNOME from networks list 😢</li>
-                <li>Fix user-online notifications</li>
-                <li>Fix selecting existing room in mobile view</li>
-              </ul>`,
+        });
 
-            transient_for: this.active_window,
-            modal: true,
-        };
-
-        const aboutDialog = new Adw.AboutWindow(aboutParams);
         aboutDialog.show();
+        aboutDialog.present(this.activeWindow);
     }
 
     _onQuit() {
