@@ -14,17 +14,14 @@ import Gtk from 'gi://Gtk';
 import Pango from 'gi://Pango';
 import Polari from 'gi://Polari';
 import Tp from 'gi://TelepathyGLib';
-import Tpl from 'gi://TelepathyLogger';
 
 import {DropTargetIface} from './pasteManager.js';
+import {LogWalker} from './logger.js';
 import {UserPopover} from './userList.js';
 import UserStatusMonitor from './userTracker.js';
 import URLPreview from './urlPreview.js';
 import * as Utils from './utils.js';
 
-Gio._promisify(Tpl.LogWalker.prototype,
-    'get_events_async', 'get_events_finish');
-
 export const MAX_NICK_CHARS = 8;
 const IGNORE_STATUS_TIME = 5;
 
@@ -345,6 +342,18 @@ class ChatView extends Gtk.ScrolledWindow {
                 this._autoscroll = true;
         });
 
+        this._queriedInitialBacklog = false;
+        this.connect('map', () => {
+            if (this._app.importingLogs)
+                return;
+
+            if (!this._queriedInitialBacklog) {
+                this._queriedInitialBacklog = true;
+                this._fetchingBacklog = true;
+                this._getLogEvents(NUM_INITIAL_LOG_EVENTS);
+            }
+        });
+
         this.vadjustment.connect('value-changed',
             this._onValueChanged.bind(this));
         this.vadjustment.connect('changed', this._updateScroll.bind(this));
@@ -388,7 +397,6 @@ class ChatView extends Gtk.ScrolledWindow {
         this._pending = new Map();
         this._pendingLogs = [];
         this._initialPending = [];
-        this._backlogTimeoutId = 0;
         this._statusCount = {left: 0, joined: 0, total: 0};
 
         this._activeNickColor = new Gdk.RGBA();
@@ -404,18 +412,7 @@ class ChatView extends Gtk.ScrolledWindow {
         });
         this._updateMaxNickChars(this._room.account.nickname.length);
 
-        let isRoom = room.type === Tp.HandleType.ROOM;
-        let target = new Tpl.Entity({
-            type: isRoom ? Tpl.EntityType.ROOM : Tpl.EntityType.CONTACT,
-            identifier: room.channel_name,
-        });
-        let logManager = Tpl.LogManager.dup_singleton();
-        this._logWalker = logManager.walk_filtered_events(
-            room.account, target,
-            Tpl.EventTypeMask.TEXT, null);
-
-        this._fetchingBacklog = true;
-        this._getLogEvents(NUM_INITIAL_LOG_EVENTS);
+        this._logWalker = new LogWalker(this._room);
 
         this._autoscroll = true;
         this._originalUpper = this.vadjustment.get_upper();
@@ -568,10 +565,6 @@ class ChatView extends Gtk.ScrolledWindow {
             this._app.disconnect(this._roomFocusChangedId);
         this._roomFocusChangedId = 0;
 
-        if (this._backlogTimeoutId)
-            GLib.source_remove(this._backlogTimeoutId);
-        this._backlogTimeoutId = 0;
-
         if (this._nickStatusChangedId) {
             this._userTracker.unwatchRoomStatus(
                 this._room, this._nickStatusChangedId);
@@ -579,54 +572,44 @@ class ChatView extends Gtk.ScrolledWindow {
         this._nickStatusChangedId = 0;
         this._userTracker = null;
 
-        this._logWalker.run_dispose();
         this._logWalker = null;
     }
 
     async _getLogEvents(num) {
-        const [events] = await this._logWalker.get_events_async(num);
+        try {
+            const events = await this._logWalker.getEvents(num);
 
-        this._hideLoadingIndicator();
-        this._fetchingBacklog = false;
+            this._hideLoadingIndicator();
+            this._fetchingBacklog = false;
 
-        let messages = events.map(e => this._createMessage(e));
-        this._pendingLogs = messages.concat(this._pendingLogs);
-        this._insertPendingLogs();
+            this._pendingLogs = events.concat(this._pendingLogs);
+            this._insertPendingLogs();
+        } catch (e) {
+            console.debug(e);
+        }
     }
 
     _createMessage(source) {
         if (source instanceof Tp.Message) {
-            let [text] = source.to_text();
-            let [id, valid] = source.get_pending_message_id();
-            return {
-                nick: source.sender.alias,
-                text,
-                timestamp: source.get_sent_timestamp() ||
-                           source.get_received_timestamp(),
-                messageType: source.get_message_type(),
-                pendingId: valid ? id : undefined,
-            };
-        } else if (source instanceof Tpl.Event) {
-            return {
-                nick: source.sender.alias,
-                text: source.message,
-                timestamp: source.timestamp,
-                messageType: source.get_message_type(),
-            };
+            const [id, valid] = source.get_pending_message_id();
+            const msg = Polari.Message.new_from_tp_message(source);
+            msg.pendingId = valid ? id : undefined;
+            return msg;
         }
 
         throw new Error(`Cannot create message from source ${source}`);
     }
 
     _getReadyLogs() {
-        if (this._logWalker.is_end())
+        if (this._logWalker.isEnd())
             return this._pendingLogs.splice(0);
 
-        let {nick, messageType: type} = this._pendingLogs[0];
-        let maxNum = this._pendingLogs.length - this._initialPending.length;
+        const nick = this._pendingLogs[0].get_sender();
+        const isAction = this._pendingLogs[0].is_action();
+        const maxNum = this._pendingLogs.length - this._initialPending.length;
         for (let i = 0; i < maxNum; i++) {
-            if (this._pendingLogs[i].nick !== nick ||
-                this._pendingLogs[i].messageType !== type)
+            if (this._pendingLogs[i].get_sender() !== nick ||
+                this._pendingLogs[i].is_action() !== isAction)
                 return this._pendingLogs.splice(i);
         }
         return [];
@@ -639,10 +622,10 @@ class ChatView extends Gtk.ScrolledWindow {
         let numLogs = logs.length;
         let pos;
         for (pos = numLogs - pending.length; pos < numLogs; pos++) {
-            if (logs[pos].nick === firstPending.nick &&
-                logs[pos].text === firstPending.text &&
-                logs[pos].timestamp === firstPending.timestamp &&
-                logs[pos].messageType === firstPending.messageType)
+            if (logs[pos].get_sender() === firstPending.get_sender() &&
+                logs[pos].get_text() === firstPending.get_text() &&
+                logs[pos].is_action() === firstPending.is_action() &&
+                logs[pos].get_time().equal(firstPending.get_time()))
                 break;
         }
         // Remove entries that are also in pending (if any), then
@@ -748,8 +731,11 @@ class ChatView extends Gtk.ScrolledWindow {
     }
 
     _fetchBacklog() {
+        if (this._app.importingLogs)
+            return Gdk.EVENT_PROPAGATE;
+
         if (this.vadjustment.value !== 0 ||
-            this._logWalker.is_end())
+            this._logWalker.isEnd())
             return Gdk.EVENT_PROPAGATE;
 
         if (this._fetchingBacklog)
@@ -757,11 +743,7 @@ class ChatView extends Gtk.ScrolledWindow {
 
         this._fetchingBacklog = true;
         this._showLoadingIndicator();
-        this._backlogTimeoutId = GLib.timeout_add(GLib.PRIORITY_DEFAULT, 500, () => {
-            this._getLogEvents(NUM_LOG_EVENTS);
-            this._backlogTimeoutId = 0;
-            return GLib.SOURCE_REMOVE;
-        });
+        this._getLogEvents(NUM_LOG_EVENTS);
         return Gdk.EVENT_STOP;
     }
 
@@ -1285,42 +1267,44 @@ class ChatView extends Gtk.ScrolledWindow {
         let iter = this._view.buffer.get_end_iter();
         this._insertMessage(iter, message, this._state);
 
-        if (message.pendingId === undefined /* outgoing */ ||
-            this._app.isRoomFocused(this._room) && this._pending.size === 0)
+        if (message.is_self() /* outgoing */ ||
+            (this._app.isRoomFocused(this._room) && this._pending.size === 0))
             this._channel.ack_message_async(tpMessage, null);
         else if (this._needsIndicator)
             this._setIndicatorMark(this._view.buffer.get_end_iter());
     }
 
     _insertMessage(iter, message, state) {
-        let isAction = message.messageType === Tp.ChannelTextMessageType.ACTION;
-        let needsGap = message.nick !== state.lastNick || isAction;
-        let highlight = this._room.should_highlight_message(
-            message.nick, message.text);
+        const nick = message.get_sender();
+        let text = message.get_text();
+        const isAction = message.is_action();
+        let needsGap = nick !== state.lastNick || isAction;
+        const highlight = this._room.should_highlight_message(nick, text);
+        const timestamp = message.get_time().to_unix();
 
-        if (message.timestamp - TIMESTAMP_INTERVAL > state.lastTimestamp) {
+        if (timestamp - TIMESTAMP_INTERVAL > state.lastTimestamp) {
             let tags = [this._lookupTag('timestamp')];
             if (needsGap)
                 tags.push(this._lookupTag('gap'));
             needsGap = false;
             this._insertWithTags(iter,
-                `${this._formatTimestamp(message.timestamp)}\n`, tags);
+                `${this._formatTimestamp(timestamp)}\n`, tags);
         }
-        state.lastTimestamp = message.timestamp;
+        state.lastTimestamp = timestamp;
 
-        this._updateMaxNickChars(message.nick.length);
+        this._updateMaxNickChars(nick.length);
 
         let tags = [];
         if (isAction) {
-            message.text = `${message.nick} ${message.text}`;
+            text = `${nick} ${text}`;
             state.lastNick = null;
             tags.push(this._lookupTag('action'));
             if (needsGap)
                 tags.push(this._lookupTag('gap'));
         } else {
-            if (state.lastNick !== message.nick) {
+            if (state.lastNick !== nick) {
                 let nickTags = [this._lookupTag('nick')];
-                let nickTagName = this._getNickTagName(message.nick);
+                let nickTagName = this._getNickTagName(nick);
                 let nickTag = this._lookupTag(nickTagName);
                 let buffer = this._view.get_buffer();
 
@@ -1328,7 +1312,7 @@ class ChatView extends Gtk.ScrolledWindow {
                     nickTag = new ButtonTag({name: nickTagName});
                     nickTag.connect('clicked', this._onNickTagClicked.bind(this));
 
-                    let status = this._userTracker.getNickRoomStatus(message.nick, this._room);
+                    let status = this._userTracker.getNickRoomStatus(nick, this._room);
                     this._updateNickTag(nickTag, status);
 
                     buffer.get_tag_table().add(nickTag);
@@ -1345,10 +1329,10 @@ class ChatView extends Gtk.ScrolledWindow {
 
                 if (needsGap)
                     nickTags.push(this._lookupTag('gap'));
-                this._insertWithTags(iter, message.nick, nickTags);
+                this._insertWithTags(iter, nick, nickTags);
                 buffer.insert(iter, '\t', -1);
             }
-            state.lastNick = message.nick;
+            state.lastNick = nick;
             tags.push(this._lookupTag('message'));
         }
 
@@ -1358,8 +1342,6 @@ class ChatView extends Gtk.ScrolledWindow {
         let params = this._room.account.dup_parameters_vardict().deep_unpack();
         let server = params.server.deep_unpack();
 
-        let {text} = message;
-
         // mask identify passwords in private chats
         if (this._room.type === Tp.HandleType.CONTACT) {
             let [isIdentify, command_, username_, password] =
