@@ -1,3 +1,9 @@
+// SPDX-FileCopyrightText: 2013 Florian Müllner <fmuellner@gnome.org>
+// SPDX-FileCopyrightText: 2015 Bastian Ilsø <bastianilso@gnome.org>
+// SPDX-FileCopyrightText: 2016 Kunaal Jain <kunaalus@gmail.com>
+//
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 import Gdk from 'gi://Gdk';
 import GdkPixbuf from 'gi://GdkPixbuf';
 import Gio from 'gi://Gio';
@@ -6,7 +12,7 @@ import GObject from 'gi://GObject';
 import Gtk from 'gi://Gtk';
 import Tp from 'gi://TelepathyGLib';
 
-import { DropTargetIface, gtypeFromFormats } from './pasteManager.js';
+import {DropTargetIface, gtypeFromFormats} from './pasteManager.js';
 import IrcParser from './ircParser.js';
 import TabCompletion from './tabCompletion.js';
 
@@ -24,9 +30,9 @@ class ChatEntry extends Gtk.Entry {
     };
 
     static [GObject.signals] = {
-        'text-pasted': { param_types: [GObject.TYPE_STRING, GObject.TYPE_INT] },
-        'image-pasted': { param_types: [GdkPixbuf.Pixbuf.$gtype] },
-        'file-pasted': { param_types: [Gio.File.$gtype] },
+        'text-pasted': {param_types: [GObject.TYPE_STRING, GObject.TYPE_INT]},
+        'image-pasted': {param_types: [GdkPixbuf.Pixbuf.$gtype]},
+        'file-pasted': {param_types: [Gio.File.$gtype]},
     };
 
     constructor(params) {
@@ -67,7 +73,7 @@ class ChatEntry extends Gtk.Entry {
         editable.stop_emission_by_name('paste-clipboard');
         const clipboard = this.get_clipboard();
 
-        const { formats } = clipboard;
+        const {formats} = clipboard;
         const type = gtypeFromFormats(formats);
         const value = await this._readClipboardValue(clipboard, type);
         if (typeof value === 'string')
@@ -162,6 +168,10 @@ class EntryArea extends Gtk.Stack {
             'max-nick-chars', 'max-nick-chars', 'max-nick-chars',
             GObject.ParamFlags.WRITABLE,
             0, GLib.MAXUINT32, 0),
+        'confirmation-visible': GObject.ParamSpec.boolean(
+            'confirmation-visible', 'confirmation-visible', 'confirmation-visible',
+            GObject.ParamFlags.READABLE,
+            false),
     };
 
     static get _nickPopover() {
@@ -174,13 +184,16 @@ class EntryArea extends Gtk.Stack {
     _popoverClosedId = 0;
 
     constructor(params) {
-        const { room } = params;
+        const {room} = params;
         delete params.room;
 
         super(params);
 
         this._room = room;
 
+        this.connect('notify::visible-child-name',
+            () => this.notify('confirmation-visible'));
+
         this.connect('destroy', this._onDestroy.bind(this));
         this.connect('notify::sensitive', this._onSensitiveChanged.bind(this));
         this.connect('realize', () => {
@@ -290,6 +303,19 @@ class EntryArea extends Gtk.Stack {
         this._updateNick();
     }
 
+    // eslint-disable-next-line camelcase
+    get confirmation_visible() {
+        return this.visible_child_name === 'paste-confirmation';
+    }
+
+    _setToolbarStyle(style) {
+        if (this._toolbarStyle === style)
+            return;
+
+        this._toolbarStyle = style;
+        this.notify('toolbar-style');
+    }
+
     _updateCompletions() {
         let nicks = [];
 
@@ -304,7 +330,7 @@ class EntryArea extends Gtk.Stack {
     }
 
     _canFocusChatEntry() {
-        const toplevelFocus = this._chatEntry.get_root().get_focus();
+        const toplevelFocus = this._chatEntry.get_root()?.get_focus();
         return this.sensitive &&
                this._chatEntry.get_mapped() &&
                !this._chatEntry['has-focus'] &&
@@ -473,7 +499,7 @@ class EntryArea extends Gtk.Stack {
     }
 
     _updateNick() {
-        let { channel } = this._room || {};
+        let {channel} = this._room || {};
         let nick = '';
         if (channel)
             nick = channel.connection.self_contact.alias;
