@@ -8,6 +8,7 @@
 #pragma once
 
 #include <glib.h>
+#include <libtracker-sparql/tracker-sparql.h>
 
 char *polari_util_get_basenick (const char *nick);
 
@@ -18,3 +19,7 @@ gboolean polari_util_match_identify_message (const char  *message,
                                              char       **command,
                                              char       **username,
                                              char       **password);
+
+TrackerSparqlConnection *polari_util_get_tracker_connection (GError **error);
+
+void polari_util_close_tracker_connection (void);
