@@ -1,3 +1,9 @@
+// SPDX-FileCopyrightText: 2014 Florian Müllner <fmuellner@gnome.org>
+// SPDX-FileCopyrightText: 2015 Bastian Ilsø <bastianilso@gnome.org>
+// SPDX-FileCopyrightText: 2019 Daronion <stefanosdimos.98@gmail.com>
+//
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 import Adw from 'gi://Adw';
 import Gio from 'gi://Gio';
 import GLib from 'gi://GLib';
@@ -23,7 +29,7 @@ class RoomStack extends Gtk.Stack {
     constructor(params) {
         super(params);
 
-        this._sizeGroup = new Gtk.SizeGroup({ mode: Gtk.SizeGroupMode.VERTICAL });
+        this._sizeGroup = new Gtk.SizeGroup({mode: Gtk.SizeGroupMode.VERTICAL});
         this._rooms = new Map();
 
         this._roomManager = RoomManager.getDefault();
@@ -139,7 +145,7 @@ class MessageInfoBar extends Gtk.InfoBar {
         };
         super(Object.assign(defaultParams, params));
 
-        let box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL });
+        let box = new Gtk.Box({orientation: Gtk.Orientation.VERTICAL});
         this.add_child(box);
 
         this._titleLabel = new Gtk.Label({
@@ -175,7 +181,7 @@ class SavePasswordConfirmationBar extends MessageInfoBar {
         let subtitle = vprintf(
             _('Identification will happen automatically the next time you connect to %s'),
             room.account.display_name);
-        super({ title, subtitle });
+        super({title, subtitle});
 
         this._room = room;
 
@@ -210,7 +216,7 @@ class SavePasswordConfirmationBar extends MessageInfoBar {
 const ChannelErrorBar = GObject.registerClass(
 class ChannelErrorBar extends MessageInfoBar {
     constructor(room) {
-        super({ title: _('Failed to join the room') });
+        super({title: _('Failed to join the room')});
 
         this._room = room;
 
@@ -276,7 +282,7 @@ class ChatPlaceholder extends Gtk.Overlay {
         });
         this.set_child(statusPage);
 
-        const inputPlaceholder = new Gtk.Box({ valign: Gtk.Align.END });
+        const inputPlaceholder = new Gtk.Box({valign: Gtk.Align.END});
         sizeGroup.add_widget(inputPlaceholder);
         this.add_overlay(inputPlaceholder);
     }
@@ -285,10 +291,10 @@ class ChatPlaceholder extends Gtk.Overlay {
 const RoomView = GObject.registerClass(
 class RoomView extends Gtk.Overlay {
     constructor(room, sizeGroup) {
-        super({ name: `RoomView ${room.display_name}` });
+        super({name: `RoomView ${room.display_name}`});
 
-        let box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL });
-        this.set_child(box);
+        const toolbarView = new Adw.ToolbarView();
+        this.set_child(toolbarView);
 
         if (room.type === Tp.HandleType.CONTACT)
             this.add_overlay(new SavePasswordConfirmationBar(room));
@@ -296,13 +302,21 @@ class RoomView extends Gtk.Overlay {
         this.add_overlay(new ChannelErrorBar(room));
 
         this._view = new ChatView(room);
-        box.append(this._view);
+        toolbarView.content = this._view;
 
         this._entryArea = new EntryArea({
             room,
             sensitive: false,
         });
-        box.append(this._entryArea);
+        toolbarView.add_bottom_bar(this._entryArea);
+        this._entryArea.bind_property_full('confirmation-visible',
+            toolbarView, 'bottom-bar-style',
+            GObject.BindingFlags.SYNC_CREATE,
+            (v, source) => [
+                true,
+                source ? Adw.ToolbarStyle.RAISED : Adw.ToolbarStyle.FLAT,
+            ],
+            null);
 
         this._view.bind_property('max-nick-chars',
             this._entryArea, 'max-nick-chars',
