@@ -1,39 +1,43 @@
-export { TabCompletion };
+// SPDX-FileCopyrightText: 2013 Florian Müllner <fmuellner@gnome.org>
+// SPDX-FileCopyrightText: 2014 Carlos Garnacho <carlosg@gnome.org>
+// SPDX-FileCopyrightText: 2016 Kunaal Jain <kunaalus@gmail.com>
+//
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 import Gdk from 'gi://Gdk';
 import Gtk from 'gi://Gtk';
-import Pango from 'gi://Pango';
 
 import * as IrcParser from './ircParser.js';
 
-const TabCompletion = class {
+export default class TabCompletion {
     constructor(entry) {
         this._entry = entry;
         this._canComplete = false;
         this._key = '';
 
-        this._entry.connect('key-press-event', this._onKeyPress.bind(this));
-        this._entry.connect('focus-out-event', this._cancel.bind(this));
-        this._entry.connect('unmap', this._cancel.bind(this));
-        this._entry.connect('realize', () => {
-            this._popup.set_transient_for(this._entry.get_toplevel());
+        this._keyController = new Gtk.EventControllerKey({
+            propagation_phase: Gtk.PropagationPhase.CAPTURE,
         });
+        this._keyController.connect('key-pressed', this._onKeyPressed.bind(this));
+        this._entry.add_controller(this._keyController);
 
-        this._popup = new Gtk.Window({ type: Gtk.WindowType.POPUP });
+        this._focusController = new Gtk.EventControllerFocus();
+        this._focusController.connect('leave', this._cancel.bind(this));
+        this._entry.add_controller(this._focusController);
 
-        // HACK: tooltips are the only popup windows that don't require a
-        //       grab on wayland
-        this._popup.set_type_hint(Gdk.WindowTypeHint.TOOLTIP);
+        this._entry.connect('unmap', this._cancel.bind(this));
 
-        let frame = new Gtk.Frame({ visible: true });
-        this._popup.add(frame);
+        this._popup = new Gtk.Popover({
+            position: Gtk.PositionType.TOP,
+            autohide: false,
+        });
+        this._popup.set_parent(this._entry);
 
-        this._list = new Gtk.ListBox({ selection_mode: Gtk.SelectionMode.SINGLE });
+        this._list = new Gtk.ListBox({selection_mode: Gtk.SelectionMode.SINGLE});
         this._list.set_filter_func(this._filter.bind(this));
         this._list.connect('row-selected', this._onRowSelected.bind(this));
         this._list.connect('row-activated', this._stop.bind(this));
-        this._list.connect('keynav-failed', this._onKeynavFailed.bind(this));
-        frame.add(this._list);
+        this._popup.set_child(this._list);
 
         this._widgetMap = new Map();
         this._previousWasCommand = false;
@@ -43,38 +47,25 @@ const TabCompletion = class {
             let row = new Gtk.ListBoxRow();
             row._text = `/${commands[i]}`;
             row._casefoldedText = row._text.toLowerCase();
-            row.add(new Gtk.Label({
+            row.set_child(new Gtk.Label({
                 label: row._text,
                 halign: Gtk.Align.START,
                 margin_start: 6,
                 margin_end: 6,
             }));
-            this._list.add(row);
+            this._list.append(row);
         }
     }
 
     _showPopup() {
-        this._list.show_all();
+        const delegate = this._entry.get_delegate();
+        const [extents] = delegate.compute_cursor_extents(this._startPos);
+        const [, bounds] = this._entry.compute_bounds(delegate);
 
-        let [, height] = this._list.get_preferred_height();
-        let [, width] = this._list.get_preferred_width();
-        this._popup.resize(width, height);
-
-        let win = this._entry.get_window();
-
-        let layout = this._entry.get_layout();
-        let layoutIndex = this._entry.text_index_to_layout_index(this._startPos);
-        let wordPos = layout.index_to_pos(layoutIndex);
-        let [layoutX] = this._entry.get_layout_offsets();
-
-        let allocation = this._entry.get_allocation();
-        let [ret_, x, y] = win.get_origin();
-        x += allocation.x + Math.min(
-            (layoutX + wordPos.x) / Pango.SCALE,
-            allocation.width - width);
-        y += allocation.y - height;
-        this._popup.move(x, y);
-        this._popup.show();
+        this._popup.pointing_to = new Gdk.Rectangle({
+            x: extents.get_x() - bounds.get_x(),
+        });
+        this._popup.popup();
     }
 
     setCompletions(completions) {
@@ -99,7 +90,7 @@ const TabCompletion = class {
                 row = new Gtk.ListBoxRow();
                 row._text = nick;
                 row._casefoldedText = row._text.toLowerCase();
-                row.add(new Gtk.Label({
+                row.set_child(new Gtk.Label({
                     label: row._text,
                     halign: Gtk.Align.START,
                     margin_start: 6,
@@ -112,21 +103,21 @@ const TabCompletion = class {
         this._widgetMap = widgetMap;
 
         // All remaining rows except those with IRC commands are going unused
-        this._list.foreach(r => {
-            if (!r._text.startsWith('/'))
-                r.destroy();
+        [...this._list].forEach(r => {
+            if (r._text.startsWith('/'))
+                return;
+            this._list.remove(r);
+            r.run_dispose();
         });
 
         for (let i = 0; i < completions.length; i++) {
             let row = this._widgetMap.get(completions[i]);
-            this._list.add(row);
+            this._list.append(row);
         }
         this._canComplete = completions.length > 0;
     }
 
-    _onKeyPress(w, event) {
-        let [, keyval] = event.get_keyval();
-
+    _onKeyPressed(controller, keyval) {
         if (this._key.length === 0) {
             if (keyval === Gdk.KEY_Tab) {
                 this._start();
@@ -138,11 +129,11 @@ const TabCompletion = class {
         switch (keyval) {
         case Gdk.KEY_Tab:
         case Gdk.KEY_Down:
-            this._moveSelection(Gtk.MovementStep.DISPLAY_LINES, 1);
+            this._moveSelection(1);
             return Gdk.EVENT_STOP;
         case Gdk.KEY_ISO_Left_Tab:
         case Gdk.KEY_Up:
-            this._moveSelection(Gtk.MovementStep.DISPLAY_LINES, -1);
+            this._moveSelection(-1);
             return Gdk.EVENT_STOP;
         case Gdk.KEY_Escape:
             this._cancel();
@@ -218,7 +209,7 @@ const TabCompletion = class {
 
         this._list.invalidate_filter();
 
-        let visibleRows = this._list.get_children().filter(c => c.get_child_visible());
+        let visibleRows = [...this._list].filter(c => c.get_child_visible());
         let nVisibleRows = visibleRows.length;
 
         if (nVisibleRows === 0)
@@ -233,19 +224,12 @@ const TabCompletion = class {
         }
     }
 
-    _onKeynavFailed(w, dir) {
-        if (this._inHandler)
-            return Gdk.EVENT_PROPAGATE;
-        let count = dir === Gtk.DirectionType.DOWN ? -1 : 1;
-        this._inHandler = true;
-        this._moveSelection(Gtk.MovementStep.BUFFER_ENDS, count);
-        this._inHandler = false;
-        return Gdk.EVENT_STOP;
-    }
-
-    _moveSelection(movement, count) {
-        this._list.emit('move-cursor', movement, count);
-        let row = this._list.get_focus_child();
+    _moveSelection(count) {
+        const rows = [...this._list].filter(c => c.get_child_visible());
+        const current = this._list.get_selected_row();
+        const index = current ? rows.findIndex(r => r === current) : 0;
+        const newIndex = (index + rows.length + count) % rows.length;
+        const row = rows[newIndex];
         this._list.select_row(row);
     }
 
@@ -253,7 +237,7 @@ const TabCompletion = class {
         if (this._key.length === 0)
             return;
 
-        this._popup.hide();
+        this._popup.popdown();
         this._popup.set_size_request(-1, -1);
 
         this._key = '';
@@ -269,4 +253,4 @@ const TabCompletion = class {
         this._insertCompletion('');
         this._stop();
     }
-};
+}
