@@ -1,24 +1,9 @@
-/*
- * Copyright (c) 2011 Red Hat, Inc.
- *
- * Polari is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2 of the License, or (at your
- * option) any later version.
- *
- * Polari is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with Polari; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
- * Authors: Cosimo Cecchi <cosimoc@redhat.com>
- *          Florian Müllner <fmuellner@gnome.org>
- *
- */
+// SPDX-FileCopyrightText: 2013 Cosimo Cecchi <cosimoc@redhat.com>
+// SPDX-FileCopyrightText: 2013 Florian Müllner <fmuellner@gnome.org>
+// SPDX-FileCopyrightText: 2015 Jonas Danielsson <jonas@threetimestwo.org>
+// SPDX-FileCopyrightText: 2016 Kunaal Jain <kunaalus@gmail.com>
+//
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 import Adw from 'gi://Adw';
 import Gdk from 'gi://Gdk';
@@ -26,37 +11,7 @@ import Gio from 'gi://Gio';
 import GLib from 'gi://GLib';
 import Gtk from 'gi://Gtk';
 import Secret from 'gi://Secret';
-
-import gi from 'gi';
-let Soup;
-
-try {
-    Soup = gi.require('Soup', '3.0');
-} catch (e) {
-    Soup = gi.require('Soup', '2.4');
-
-    Soup.Message.new_from_encoded_form =
-        function (method, uri, form) {
-            const message = Soup.Message.new_from_uri(method, new Soup.URI(uri));
-            message.set_request(
-                Soup.FORM_MIME_TYPE_URLENCODED,
-                Soup.MemoryUse.COPY,
-                form);
-            return message;
-        };
-
-    Soup.Session.prototype.send_and_read_async =
-        function (message, prio, cancellable, callback) {
-            this.queue_message(message, () => callback(this, message));
-        };
-    Soup.Session.prototype.send_and_read_finish =
-        function (message) {
-            if (message.status_code !== Soup.KnownStatusCode.OK)
-                return null;
-
-            return message.response_body.flatten().get_as_bytes();
-        };
-}
+import Soup from 'gi://Soup?version=3.0';
 
 Gio._promisify(Secret, 'password_store', 'password_store_finish');
 Gio._promisify(Secret, 'password_lookup', 'password_lookup_finish');
@@ -67,11 +22,11 @@ Gio._promisify(Soup.Session.prototype,
 const SECRET_SCHEMA_ACCOUNT = new Secret.Schema(
     'org.gnome.Polari.Account',
     Secret.SchemaFlags.NONE,
-    { 'account-id': Secret.SchemaAttributeType.STRING });
+    {'account-id': Secret.SchemaAttributeType.STRING});
 const SECRET_SCHEMA_IDENTIFY = new Secret.Schema(
     'org.gnome.Polari.Identify',
     Secret.SchemaFlags.NONE,
-    { 'account-id': Secret.SchemaAttributeType.STRING });
+    {'account-id': Secret.SchemaAttributeType.STRING});
 
 const GPASTE_BASEURL = 'https://paste.gnome.org/';
 
@@ -193,7 +148,7 @@ export function storeIdentifyPassword(account, password) {
 }
 
 async function _storePassword(schema, label, account, password) {
-    let attr = { 'account-id': account.get_path_suffix() };
+    let attr = {'account-id': account.get_path_suffix()};
     let coll = Secret.COLLECTION_DEFAULT;
     try {
         await Secret.password_store(schema, attr, coll, label, password, null);
@@ -224,7 +179,7 @@ export function lookupIdentifyPassword(account) {
 }
 
 async function _lookupPassword(schema, account) {
-    let attr = { 'account-id': account.get_path_suffix() };
+    let attr = {'account-id': account.get_path_suffix()};
     let password = null;
     try {
         password = await Secret.password_lookup(schema, attr, null);
@@ -255,7 +210,7 @@ export function clearIdentifyPassword(account) {
 }
 
 async function _clearPassword(schema, account) {
-    let attr = { 'account-id': account.get_path_suffix() };
+    let attr = {'account-id': account.get_path_suffix()};
     try {
         await Secret.password_clear(schema, attr, null);
     } catch (e) {
@@ -282,7 +237,7 @@ export function findUrls(str) {
     while ((match = _urlRegexp.exec(str))) {
         let name = match[2];
         let url = GLib.uri_parse_scheme(name) ? name : `http://${name}`;
-        res.push({ name, url, pos: match.index + match[1].length });
+        res.push({name, url, pos: match.index + match[1].length});
     }
     return res;
 }
@@ -443,7 +398,7 @@ export async function imgurPaste(pixbuf, title) {
 }
 
 function checkResponse(message) {
-    const { statusCode } = message;
+    const {statusCode} = message;
     const phrase = Soup.Status.get_phrase(statusCode);
     if (statusCode !== Soup.Status.OK)
         throw new Error(`Unexpected response: ${phrase}`);
